"""
Class handling requests to remote resources.
"""
import contextlib
import urllib.request
from http import HTTPStatus

from podmena.utils import _info


class SimpleFetcher(object):
    def __init__(self, url):
        self.url = url

    def request(self, url=None):
        if url is None:
            url = self.url
        _info(f'Requesting {url}')

        req = urllib.request.Request(url)
        with contextlib.closing(urllib.request.urlopen(req)) as resp:
            if resp.status != HTTPStatus.OK:
                raise RuntimeError('Got wrong status %s' % resp.status)
            return resp.read().decode('utf-8')



import os
import sys
import shutil

import click

from podmena.fetcher import SimpleFetcher
from podmena.parser import RegexParser
from podmena.utils import (
    _warn,
    _note,
    _info,
    force_symlink,
    get_git_root_dir,
    set_git_global_hooks_path,
    get_git_config_hooks_value,
    unset_git_global_hooks_path,
)


CURRENT_DIR = os.path.dirname(os.path.realpath(__file__))
RESOURCES_DIR = os.path.join(CURRENT_DIR, 'resources')
HOOK_FILENAME = 'commit-msg'
DATABASE_FILE = 'emoji-db'


@click.group()
@click.version_option()
def cli():
    pass


@cli.command(
    name='status',
    help='Shows whether podmena is installed for current repository '
         'or globally'
)
def status():
    active = False

    git_root_dir = get_git_root_dir()
    if git_root_dir is not None:
        local_hooks_path = os.path.join(git_root_dir, '.git', 'hooks')
        database_path = os.path.join(local_hooks_path, DATABASE_FILE)
        hook_path = os.path.join(local_hooks_path, HOOK_FILENAME)
        if os.path.exists(database_path) and os.path.exists(hook_path):
            _note('podmena is activated for current repository!')
            active = True

    global_hooks_path = os.path.expanduser('~/.podmena/hooks')
    global_database_path = os.path.join(global_hooks_path, DATABASE_FILE)
    global_hook_path = os.path.join(global_hooks_path, HOOK_FILENAME)
    git_global_hooks_config = get_git_config_hooks_value()

    if (os.path.exists(global_database_path) and
        os.path.exists(global_hook_path) and
        git_global_hooks_config == global_hooks_path
    ):
        _note('podmena is activated globally!')
        active = True

    if not active:
        _warn('podmena is not activated neither for current repository '
              'nor globally!')


@cli.command(
    name='update',
    help='Update database of emoji in case of any changes from remote'
)
def grab():
    url = 'https://www.webpagefx.com/tools/emoji-cheat-sheet/'
    fetcher = SimpleFetcher(url=url)
    parser = RegexParser()
    html = fetcher.request()
    emoji = parser.parse(html)
    database_path = os.path.join(RESOURCES_DIR, DATABASE_FILE)
    with open(database_path, 'w') as f:
        f.write('\n'.join(emoji))
    _note('Downloaded {} emoji to database'.format(len(emoji)))


@cli.group(
    name='add',
    help='Activate podmena',
)
def install():
    pass


@install.command(
    name='local',
    help='Install podmena for current git repository',
)
def local_install():
    git_local_root = os.path.join(os.getcwd(), '.git')
    local_hooks_path = os.path.join(git_local_root, 'hooks')
    if os.path.exists(git_local_root) and os.path.isdir(git_local_root):
        src_file = os.path.join(RESOURCES_DIR, HOOK_FILENAME)
        dst_file = os.path.join(local_hooks_path, HOOK_FILENAME)
        shutil.copyfile(src_file, dst_file)
        os.chmod(dst_file, 0o0775)
        db_file = os.path.join(RESOURCES_DIR, DATABASE_FILE)
        db_link = os.path.join(local_hooks_path, DATABASE_FILE)
        force_symlink(db_file, db_link)
        _note('Successfully installed for current repository!')
    else:
        _warn('Not a git repository')
        sys.exit(1)


@install.command(
    name='global',
    help='Install podmena globally for every git repository',
)
def global_install():
    global_hooks_path = os.path.expanduser('~/.podmena/hooks')
    confirm_info = (
        'This will set one global hooks directory for all you repositories.\n'
        'This action may deactivate your previous hooks installed per '
        'repository.\nFor more info see '
        'https://git-scm.com/docs/git-config#git-config-corehooksPath\n'
    )
    _info(confirm_info)
    if click.confirm('Do you want to continue?', abort=True):
        if not os.path.exists(global_hooks_path):
            os.makedirs(global_hooks_path)

        src_file = os.path.join(RESOURCES_DIR, HOOK_FILENAME)
        dst_file = os.path.join(global_hooks_path, HOOK_FILENAME)
        shutil.copyfile(src_file, dst_file)
        os.chmod(dst_file, 0o0775)
        db_file = os.path.join(RESOURCES_DIR, DATABASE_FILE)
        db_link = os.path.join(global_hooks_path, DATABASE_FILE)
        force_symlink(db_file, db_link)
        set_git_global_hooks_path(global_hooks_path)
        _note('Installed globally for all repos')


@cli.group(
    name='rm',
    help='Deactivate podmena',
)
def remove():
    pass


@remove.command(
    name='local',
    help='Uninstall podmena for current git repository',
)
def local_uninstall():
    git_local_root = os.path.join(os.getcwd(), '.git')
    hook_filepath = os.path.join(git_local_root, 'hooks', HOOK_FILENAME)
    db_link = os.path.join(git_local_root, 'hooks', DATABASE_FILE)
    if os.path.exists(hook_filepath):
        os.remove(hook_filepath)
        os.remove(db_link)
        _note('Uninstalled for current repository')
    else:
        _warn('podmena is not installed for current repository!')
        sys.exit(1)


@remove.command(
    name='global',
    help='Uninstall podmena on globally',
)
def global_uninstall():
    rc = unset_git_global_hooks_path()
    if rc == 0:
        _info('Deactivated podmena globally')
    elif rc == 5:
        _warn('podmena is not installed globally!')
        sys.exit(1)
    else:
        _warn('Failed to deactivate')
        sys.exit(rc)


if __name__ == '__main__':
    cli()



import re


class RegexParser(object):
    def __init__(self):
        self.pattern = re.compile(
            r'<span .+></span>:<span .+>([\w_]+)</span>:</div>')

    def parse(self, text):
        return self.pattern.findall(text)



import os
import subprocess

import click


def _warn(message):
    click.secho(message, fg='red')


def _note(message):
    click.secho(message, fg='green')


def _info(message):
    click.secho(message, fg='yellow')


def set_git_global_hooks_path(hooks_path):
    subprocess.call([
        'git',
        'config',
        '--global',
        'core.hooksPath',
        hooks_path,
    ])


def get_git_config_hooks_value():
    try:
        return subprocess.check_output([
            'git',
            'config',
            '--get',
            'core.hooksPath',
        ]).decode().strip()
    except subprocess.CalledProcessError:
        pass


def unset_git_global_hooks_path():
    return subprocess.call([
        'git',
        'config',
        '--global',
        '--unset',
        'core.hooksPath',
    ])


def get_git_root_dir():
    try:
        return subprocess.check_output([
            'git',
            'rev-parse',
            '--show-toplevel',
        ], stderr=subprocess.STDOUT).decode().strip()
    except subprocess.CalledProcessError:
        pass


def force_symlink(src, dst):
    if os.path.exists(dst) and os.path.islink(dst):
        os.remove(dst)

    os.symlink(src, dst)



"""A setuptools based setup module.

See:
https://packaging.python.org/en/latest/distributing.html
https://github.com/pypa/sampleproject
"""

# Always prefer setuptools over distutils
from setuptools import setup, find_packages
# To use a consistent encoding
from codecs import open
from os import path


here = path.abspath(path.dirname(__file__))


with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()


setup(
    # https://packaging.python.org/specifications/core-metadata/#name
    name='podmena',

    # https://packaging.python.org/en/latest/single_source_version.html
    version='0.4.2',
    description='Enhance your git commit messages with emoji',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/bmwant/podmena',
    author='Misha Behersky',
    author_email='bmwant@gmail.com',

    # For a list of valid classifiers, see
    # https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Topic :: Utilities',
        'Topic :: Software Development :: Build Tools',
        'Topic :: Software Development :: Version Control :: Git',

        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Unix Shell',
    ],

    keywords='git hook fun emoji commit',

    packages=find_packages(exclude=['contrib', 'docs', 'tests']),

    install_requires=[
        'click>=6.7',  # hi, mitshuhiko
    ],

    package_data={
        'podmena': [
            'resources/commit-msg',
            'resources/emoji-db',
        ],
    },

    data_files=[
        ('', ['podmena/resources/emoji-db']),
    ],

    entry_points={
        'console_scripts': [
            'podmena=podmena.cli:cli',
        ],
    },

    project_urls={
        'Blog post': 'https://bmwlog.pp.ua/',
        'Say Thanks!': 'https://gimmebackmyson.herokuapp.com/',
    },
)
